package m3r3.driver;

import java.io.File;
import m3r3.driver.led.LED;
import m3r3.driver.led.LEDType;
import m3r3.driver.button.Button;
import m3r3.driver.button.ButtonType;
import m3r3.driver.servo.Servo;

public class ButtonModul{
	static {
		File lib = new File("lib/" + System.mapLibraryName("button_modul"));
		System.load(lib.getAbsolutePath());
	}
	
	public static final int NO_CHANGE			= 0x0;
	public static final int STATIC_LIGHT		= 0x1;
	public static final int RUNNING_LIGHT		= 0x2;
	public static final int RUNNING_LIGHT_BW	= 0x3;
	public static final int ROTATING_LIGHT		= 0x4;
	public static final int ROTATING_LIGHT_BW	= 0x5;
	public static final int LED_AUTO			= 0x6;
	public static final int FREE_1				= 0x7;
	public static final int SERVO_POSITION		= 0x8;
	public static final int FREE_2				= 0x9;
	public static final int FREE_3				= 0xA;
	public static final int FREE_4				= 0xB;
	public static final int FREE_5				= 0xC;
	public static final int FREE_6				= 0xD;
	public static final int FREE_7				= 0xE;
	public static final int FREE_8				= 0xF;
	
	private LED[] leds;
	private Button[] buttons;
	private Servo servo;
	private static ButtonModul instance;
	private int state;
	private int servoPosition;
	
	private ButtonModul(){
		init();
		
		leds = new LED[4];
		leds[0] = new LED(LEDType.LED_LEFT);
		leds[1] = new LED(LEDType.LED_MIDDLE);
		leds[2] = new LED(LEDType.LED_RIGHT);
		leds[3] = new LED(LEDType.LED_DISPLAY);
		
		buttons = new Button[4];
		buttons[0] = new Button(ButtonType.BUTTON_LEFT, false);
		buttons[1] = new Button(ButtonType.BUTTON_MIDDLE, false);
		buttons[2] = new Button(ButtonType.BUTTON_RIGHT, false);
		buttons[3] = new Button(ButtonType.BUTTON_SWITCH, false);
		
		servo = new Servo();
		update();
	}
	
	public static ButtonModul getInstance(){
		if(ButtonModul.instance == null){
			ButtonModul.instance = new ButtonModul();
		}
		return ButtonModul.instance;
	}
	
	private static native void init();
	
	private static native int transmit(int value);
	
	public LED getLED(LEDType type){
		for(LED led : leds){
			if(led.getType() == type){
				return led;
			}
		}
		return null;
	}
	
	public Button getButton(ButtonType type){
		for(Button button : buttons){
			if(button.getType() == type){
				return button;
			}
		}
		return null;
	}
	
	void update(){
		int transmition = state << 12;
		for(int i=0; i<4; i++){
			transmition |= (leds[i].getRed()	?1:0) << i + 0;
			transmition |= (leds[i].getGreen()	?1:0) << i + 1;
			transmition |= (leds[i].getBlue()	?1:0) << i + 2;
		}
		if(servo.getPosition() != servoPosition){
			transmit(transmition);
			servoPosition = servo.getPosition();
			transmition = (SERVO_POSITION << 12) | servoPosition;
		}
		int receive = transmit(transmition);
		for(int i=0; i<4; i++){
			buttons[i].setState((receive & (1<<i))!= 0);
		}
	}
	
	public int setState(int state){
		if(state > 0xF) return 1;
		this.state = state;
		return 0;
	}
	
	public int getState(){
		return this.state;
	}
	
	public Servo getServo(){
		return servo;
	}
}